#ifndef INTERNALS_HPP
#define INTERNALS_HPP

template<typename T>
class Buffer {
public:
  Buffer(T initial);
protected:
  T& value();
  const T& value() const;
private:
  T M_value;
};

class Register : public Buffer<int> {
public:
  Register(int initial = 0);

  int get() const;
  void set(int value);
};

class Pointer : public Register {
public:
  Pointer();
  void incr(int step = 1);
};

class Flag : public Buffer<bool> {
public:
  Flag(bool initial = true);

  bool test() const;
  void set(bool value);
};

class OutPort : public Buffer<int> {
public:
  OutPort();
  void set(int value);
};

class Internals {
public:
  Internals();

  Register& A();
  const Register& A() const;

  Register& B();
  const Register& B() const;

  Register& W();
  const Register& W() const;

  Pointer& IP();
  const Pointer& IP() const;

  Register& saved_IP();
  const Register& saved_IP() const;

  Flag& Status();
  const Flag& Status() const;
  
  OutPort& OPort();
  const OutPort& OPort() const;

private:
  Register  M_reg_A;
  Register  M_reg_B;
  Register  M_reg_W;
  Pointer   M_pointer_IP;
  Register  M_reg_saved_IP;
  Flag      M_flag_Status;
  OutPort   M_port_OPort;
};

#endif // INTERNALS_HPP
